//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Device/ResolutionForm.cpp
//! @brief     Implements class ResolutionForm
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Device/ResolutionForm.h"
#include "GUI/Model/Detector/DetectorItem.h"
#include "GUI/Model/Detector/ResolutionFunctionItems.h"
#include "GUI/Support/Style/GroupBoxCollapser.h"
#include "GUI/View/Numeric/ComboUtil.h"
#include "GUI/View/Numeric/DoubleSpinBox.h"
#include "GUI/View/Numeric/NumWidgetUtil.h"

ResolutionForm::ResolutionForm(QWidget* parent, DetectorItem* item)
    : QGroupBox("Resolution function", parent)
{
    ASSERT(item);
    auto formLayout = new QFormLayout(this);
    formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);

    auto updateResolutionForm = [parent = this, layout = formLayout, item]() -> void {
        while (layout->rowCount() > 1)
            layout->removeRow(1);

        auto* resFunction = item->resolutionFunctionSelection().currentItem();
        if (auto* p = dynamic_cast<ResolutionFunction2DGaussianItem*>(resFunction)) {
            auto* sigmaXSpinBox = GUI::Util::createDoubleSpinBoxRow(layout, p->sigmaX());
            auto* sigmaYSpinBox = GUI::Util::createDoubleSpinBoxRow(layout, p->sigmaY());

            connect(sigmaXSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
                    [parent, p](double newValue) {
                        p->setSigmaX(newValue);
                        emit parent->dataChanged();
                    });

            connect(sigmaYSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
                    [parent, p](double newValue) {
                        p->setSigmaY(newValue);
                        emit parent->dataChanged();
                    });
        }
    };

    auto* typeCombo = GUI::Util::createComboBoxFromProperty(item->resolutionFunctionSelection(),
                                                            [this, updateResolutionForm](int) {
                                                                updateResolutionForm();
                                                                emit dataChanged();
                                                            });
    formLayout->addRow("Type:", typeCombo);

    auto* collapser = GroupBoxCollapser::installIntoGroupBox(this);
    collapser->setExpanded(item->isExpandResolutionFunc());
    connect(collapser, &GroupBoxCollapser::toggled, this,
            [item](bool b) { item->setExpandResolutionFunc(b); });

    updateResolutionForm();
}
